import Dude from '../sprites/Dude.js';

class GameScene extends Phaser.Scene {
    constructor(test) {
        super({
            key: 'GameScene',
        });
        this.myconfig;
        this.level;this.levelnum;
        this.platforms;
        this.bgm;this.cursors;this.mysound;
        this.player;
    }

    create(data) {
        this.myconfig = this.cache.json.get('config');
        this.levelnum = (data.level <= this.myconfig.levels) ? data.level : 1;
        this.initWorld();
        this.createStars();
        this.initPlayer(data.score);

        this.initControlsAndCams();
        this.addSound();

        this.addCollisionHandler();
    }

    update() {
        this.player.update(this.cursors, this.mysound);
    }

    /*****************************
     *  Build the World 
     *****************************/

    initWorld() {
        this.add.image(0,0,'sky').setOrigin(0,0).setScale(2.5,1);
        this.platforms = this.physics.add.staticGroup();

        this.level = this.cache.json.get('level'+this.levelnum);
        this.level.platforms.forEach(this.createPlatform, this);
        this.level.decoration.forEach(this.addDecoration, this);
        //this.createBubbles();
    }

    createPlatform(p) {
        var newplatform = this.platforms.create(p.x,p.y, 'platform').setScale(p.scaleX,p.scaleY).refreshBody();
    }

    addDecoration(decodata) {
        this.add.image(decodata.x, decodata.y, 'decoration', decodata.frame);
    }

    //Showcase Partikel-Emitter
    createBubbles() {
        var particles = this.add.particles('seifenblase');

        var emitter = particles.createEmitter({
            x: 600,
            y: 300,
            speed: 100,
            quantity: 1,
            frequency: 10,
            gravityY: -10,
            lifespan: { min: 1000, max: 2000 },
            blendMode: 'ADD'
        });
    }

    /*****************************
     *  Create Stars to collect
     *****************************/

    createStars() {
        this.stars = this.physics.add.group({
            key: 'star',
            repeat: this.myconfig.stars-1,
            setXY: { x: 12, y: 0, stepX: 70 }
        });
        this.stars.children.iterate(function(stern) {
            stern.setBounceY(Phaser.Math.FloatBetween(0.4, 0.8));
        });
    }

    /*****************************
     * The Player
     *****************************/

    initPlayer(score) {
        this.player = new Dude(this,300,100,'dude');
        this.player.score = score;
    }

    /*****************************
     * Controls, ScoreBoard and Sound
     * Finish the Environment
     *****************************/

    initControlsAndCams() {
        this.cursors = this.input.keyboard.createCursorKeys();
        //Größere Welt als der Bildausschnitt - Kamera folgt dem Player
        this.cameras.main.startFollow(this.player);
        this.physics.world.setBounds(0,0,2000,600);
        this.cameras.main.setBounds(0,0,2000,600);
    }

    addSound() {
        this.mysound = {
            jump: this.sound.add("jump"),
            collect: this.sound.add("collect"),
        };
        if (this.myconfig.bgmusic) {
            this.bgm = this.sound.add('backgroundmusic');
            this.bgm.play();
        }
    }

    /*****************************
     * Manage the Collisions
     *****************************/

    addCollisionHandler() {
        this.physics.add.collider(this.player, this.platforms);
        this.physics.add.collider(this.stars, this.platforms);
    }


}

export default GameScene;