import Dude from '../sprites/Dude.js';

class GameScene extends Phaser.Scene {
    constructor(test) {
        super({
            key: 'GameScene',
        });
        this.myconfig;
        this.level;this.levelnum;
        this.platforms;
        this.bgm;this.cursors;this.mysound;
        this.player;
        this.stars;
        this.scoreText;this.levelText;
        this.worldcam;
    }

    create(data) {
        this.myconfig = this.cache.json.get('config');
        this.levelnum = (data.level <= this.myconfig.levels) ? data.level : 1;
        this.initWorld();
        this.createStars();
        this.initPlayer(data.score);

        this.initControlsAndCams();
        this.addScoreBoard();
        this.addSound();

        this.addCollisionHandler();
    }

    update() {
        this.player.update(this.cursors, this.mysound);
    }

    /*****************************
     *  Build the World 
     *****************************/

    initWorld() {
        this.add.image(0,0,'sky').setOrigin(0,0).setScale(2.5,1);
        this.platforms = this.physics.add.staticGroup();

        this.level = this.cache.json.get('level'+this.levelnum);
        this.level.platforms.forEach(this.createPlatform, this);
        this.level.decoration.forEach(this.addDecoration, this);
        //this.createBubbles();
    }

    createPlatform(p) {
        var newplatform = this.platforms.create(p.x,p.y, 'platform').setScale(p.scaleX,p.scaleY).refreshBody();
    }

    addDecoration(decodata) {
        this.add.image(decodata.x, decodata.y, 'decoration', decodata.frame);
    }

    //Showcase Partikel-Emitter
    createBubbles() {
        var particles = this.add.particles('seifenblase');

        var emitter = particles.createEmitter({
            x: 600,
            y: 300,
            speed: 100,
            quantity: 1,
            frequency: 10,
            gravityY: -10,
            lifespan: { min: 1000, max: 2000 },
            blendMode: 'ADD'
        });
    }

    /*****************************
     *  Create Stars to collect
     *****************************/

    createStars() {
        this.stars = this.physics.add.group({
            key: 'star',
            repeat: this.myconfig.stars-1,
            setXY: { x: 12, y: 0, stepX: 70 }
        });
        this.stars.children.iterate(function(stern) {
            stern.setBounceY(Phaser.Math.FloatBetween(0.4, 0.8));
        });

    }

    /*****************************
     * The Player
     *****************************/

    initPlayer(score) {
        this.player = new Dude(this,300,100,'dude');
        this.player.score = score;
    }

    /*****************************
     * Controls, ScoreBoard and Sound
     * Finish the Environment
     *****************************/

    initControlsAndCams() {
        this.cursors = this.input.keyboard.createCursorKeys();
        //Größere Welt als der Bildausschnitt - Kamera folgt dem Player
        this.cameras.main.startFollow(this.player);
        this.physics.world.setBounds(0,0,2000,600);
        this.cameras.main.setBounds(0,0,2000,600);
        this.addMiniMap();
    }

    addMiniMap() {
        //Just another cam!
        this.worldcam = this.cameras.add(10, 10, 400, 120).setZoom(0.2).setName('mini');
        this.worldcam.scrollX = 800;
        this.worldcam.scrollY = 240;
        //this.worldcam.setBounds(0,0,2000,600);
    }

    addScoreBoard() {
        this.scoreText = this.add.text(20, 550, 'Score: '+this.player.score, { fontSize: '32px', fill: '#FFF' });
        this.scoreText.setScrollFactor(0,0);
        this.levelText = this.add.text(400, 550, 'Level: '+this.levelnum, { fontSize: '32px', fill: '#FFF' });
        this.levelText.setScrollFactor(0,0);
    }

    addSound() {
        this.mysound = {
            jump: this.sound.add("jump"),
            collect: this.sound.add("collect"),
        };
        if (this.myconfig.bgmusic) {
            this.bgm = this.sound.add('backgroundmusic');
            this.bgm.play();
        }
    }

    /*****************************
     * Manage the Collisions
     *****************************/

    addCollisionHandler() {
        this.physics.add.collider(this.player, this.platforms);
        this.physics.add.collider(this.stars, this.platforms);

        //Collisions with Callback-Functions
        this.physics.add.overlap(this.player, this.stars, this.collectStar, null, this);
    }

    collectStar(player, star) {
        this.mysound.collect.play();
        star.destroy();
    
        //  Add and update the score
        this.player.score += 10;
        this.scoreText.setText('Score: ' + this.player.score);
        if (this.stars.countActive(true) == 0) {
            this.startNewLevel(this.levelnum,player.score);
        }
    }

    startNewLevel(level,score) {
        this.cameras.main.fade(1000,0,0,0,false);
        this.cameras.main.on("camerafadeoutcomplete",function() {
            var newLevel = level + 1;
            this.scene.restart({level:newLevel,score:score});
        },this);
    }

}

export default GameScene;